package mi.hdm.recipes;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

import java.util.ArrayList;
import java.util.List;
import java.util.Optional;

public class Categories {
    private static final Categories instance = new Categories();
    private static final Logger log = LogManager.getLogger(Categories.class);

    private final List<Category> allCategories;

    private Categories() {
        allCategories = new ArrayList<>();
    }

    public static Categories getInstance() {
        return instance;
    }

    /**
     * Adds a category if neither the given name nor the color code have been used in a category before.
     * @param name Name for the category that is to be added
     * @param colourCode Color code for the category that is to be added
     * @return True if the category has been added (meaning no equal category existed before), otherwise false.
     */
    public boolean addCategory(String name, int colourCode){
        Category c = new Category(name, colourCode);
        if(allCategories.contains(c)) {
            log.error("Category: {} not added because it already exists", c.getName());
            return false;
        } else {
            allCategories.add(c);
            log.info("Category {} added successfully.", c.getName());
            return true;
        }
    }

    public boolean deleteCategory(Category c) {
        return allCategories.remove(c);
    }

    public boolean deleteCategory(String name) {
        Category c = getCategoryByName(name).orElseThrow(() -> new RuntimeException("Category not found."));
        return allCategories.remove(c);
    }

    public List<Category> getAllCategories(){
        return allCategories;
    }

    public void clearCategories() {
        allCategories.clear();
    }

    private Optional<Category> getCategoryByName(String name) {
        return allCategories.stream()
                .filter(c -> c.getName().equals(name))
                .findFirst();
    }
}
